library(survival)
## On a besoin d'une fonction qui évalue pour un temps x la valeur d'une fonction en escalier définie par ses bins (abscisses où elle change de valeur)
## et les valeurs (values) qu'elle prend à ces abscisses
piecewise_survival_eval = function(x, bins, values){
  # x = vector where the functions has to be evaluated
  # bins : x_values where the function change of value
  # values : values of the function at x_values
  # before min(bins) the function equals 1
  n_bins = length(bins)
  n_values = length(values)
  values = c(values,values[length(values)])
  out = rep(0, length(x))
  bin_idx = 1
  i = 1
  for (x_i in x){
    if (x_i < min(bins)){
      out[i] = 1
    }
    else{
      if (x_i == min(bins)){
        out[i] = values[1]
        print(out[i])
      }
      else{
        if (bin_idx != (n_bins - 1)){
          while (x_i > bins[bin_idx]){
            bin_idx = bin_idx + 1}
        }
        out[i] = values[bin_idx-1]}
    }
    i = i + 1}
  
  return(out)}

## Ponctual Brier score
## cette fonction calcule le score de Brier au temps t
Brier_score = function(t,survival_matrix , times , censored_times, censoring_indicators){
  #t : time where the Brier score is evaluated
  #survival_matrix : matrix of size (n_ind,length(times)) giving the estimated survival at each time of times
  #times : vector where the survival functions in survival matrix have been evaluated
  #censored_times : observed times on dataset, size n_ind
  #censoring indicator : observed censoring indicator on dataset, size n_ind
  n_obs = dim(survival_matrix)[1]
  survival_matrix = cbind(rep(1,n_obs),survival_matrix)
  times = c(0,times)
  KM_censoring = survfit(Surv(censored_times,(1-censoring_indicators)) ~ 1 ) # Kaplan Meier estimation of 
  # censoring distribution
  epsilon = min(diff(times))/10
  brier_score = 0
  hat_bar_G_t = piecewise_survival_eval(t-epsilon,KM_censoring$time,KM_censoring$surv)
  for (i in c(1:n_obs)){
    if (censored_times[i]<= t & censoring_indicators[i] == 1){
      hat_bar_G_T_i = piecewise_survival_eval(censored_times[i]-epsilon,KM_censoring$time,KM_censoring$surv)
      brier_i = (0 - piecewise_survival_eval(t-epsilon,times,survival_matrix[i,]))^2 / (hat_bar_G_T_i)
      brier_score = brier_score + brier_i
    }
    if (censored_times[i] > t){
      
      brier_i = (1 - piecewise_survival_eval(t-epsilon,times,survival_matrix[i,]))^2 / (hat_bar_G_t)
      brier_score = brier_score + brier_i
    }}
  return(brier_score / n_obs)
}


## Brier score on a vector
Brier_score_vec = function(x,survival_matrix , times , censored_times, censoring_indicators){
  bs_cox = sapply(X=x,FUN = Brier_score,survival_matrix , times , censored_times ,censoring_indicators )
  return(list(x = x, score = bs_cox))
}

## Integrated Brier score
IBrier_score = function(brierscore){
  return(sum(brierscore$score * diff(c(0,brierscore$x)))/ max(brierscore$x))
}